
# ####################################################################################
#
# process Coronate .json file and output each round : results and current standing
#
# ####################################################################################



# ### TIP

# ### create a virtual environment
# python3 -m venv .venv

# ### activate the virtual environment
# source .venv/bin/activate

# ----------------------------------------------------------------------------

# ### EXAMPLE DATA

# "players": {
#   "-TO37yY5ihuGQZkzSYcQc": {
#     "firstName": "Anne",
#     "id": "-TO37yY5ihuGQZkzSYcQc",
#     "lastName": "Bergen",
#     "matchCount": 3,
#     "rating": 1453,
#     "type_": "person"
#   },
#   "0TpMnKv9R5FMHNiSUjsyd": {
#     "firstName": "Omar",
#     "id": "0TpMnKv9R5FMHNiSUjsyd",
#     "lastName": "Mahmutovic",
#     "matchCount": 1,
#     "rating": 1379,
#     "type_": "person"
#   },


# "tournaments": {
#   "2BZYvlkc4m6Gc82y5YKr7": {
#     "roundList": [
#       [
#         {
#           "id": "kI9zd6du_cmQ7S5IhbUOy",
#           "whiteId": "4s2PKcsifORXRE2RSpmzn",
#           "blackId": "0TpMnKv9R5FMHNiSUjsyd",
#           "whiteNewRating": 1800,
#           "blackNewRating": 1410,
#           "whiteOrigRating": 1800,
#           "blackOrigRating": 1410,
#           "result": "whiteWon"
#         },
#         {
#           "id": "DWMu7ELnPd1XEOXyTx20T",
#           "whiteId": "spmeo5vjjiUeOVaRkwqAj",
#           "blackId": "KuHPQpTZRG4cG_po4MfJ_",
#           "whiteNewRating": 1736,
#           "blackNewRating": 1415,
#           "whiteOrigRating": 1751,
#           "blackOrigRating": 1400,
#           "result": "draw"
#         },

# ----------------------------------------------------------------------------

import json
# from pprint import pprint
from collections import OrderedDict
import math



# ===================================================================================
# YOUR CONFIGURATION DATA
# ===================================================================================

# GTFshowRating = True
GTFshowRating = False

myJSON = "./coronate-2023-11-06-SVPS-Rapid-rounds-4-5-6-B.json"

# sHeader = "[#] name                   points games perc"
sHeader = "[#] naam                     punt games perc"



# ===================================================================================
# DO NOT EDIT BELOW THIS LINE, UNLESS YOU KNOW WHAT YOU'RE DOING ..
# ===================================================================================

GEOL = "\n"

GDTR = {  # Global Dictionary of TRanslations
	"whiteWon": "1-0",
	"blackWon": "0-1",
	'draw': "½-½"
}

dStanding = {
	# >> example data
	# "Roelof Berkepeis": {
	# 	"Npoint": 2,
	# 	"Ngame": 3,
	# 	"rating": 0
	# },
	# "Mart Wennink": {
	# 	"Npoint": 3,
	# 	"Ngame": 6,
	# 	"rating": 0
	# },
	# >> ...
}

# ---------------------------------------------------------------------------------------------------
# FUNCTIONS
# ---------------------------------------------------------------------------------------------------

def rtDplayersData(game):
	d2 = {
		"nameWhite": "",
		'nameBlack': "",
		'result': GDTR[game['result']],
		'ratingWhite': 0,
		'ratingBlack': 0
	}
	for key in d['players']:

		if key == game['whiteId']:
			d2['nameWhite'] = d['players'][key]['firstName'] + " " + d['players'][key]['lastName']
			d2['ratingWhite'] = game['whiteNewRating']

		if key == game['blackId']:
			d2['nameBlack'] = d['players'][key]['firstName'] + " " + d['players'][key]['lastName']
			d2['ratingBlack'] = game['blackNewRating']

	return d2


def rtPRC(item):
	return round(100 * item['Npoint'] / item['Ngame'])

def rtPoint(c):
	if c == '½':
		return 0.5
	return int(c)

def rtPoint2(n):
	n = round(n) if math.floor(n) == n else n
	n = str(n)
	if n == "0.5":
		n = '½'
	elif n.find(".5") == -1:
		n += " "
	else:
		n = n.replace(".5",'½')
	return n



# ---------------------------------------------------------------------------------------------------
# SCRIPT start
# > read .json file and build output
# ---------------------------------------------------------------------------------------------------

with open(myJSON) as f:
	d = json.load(f)

	for trn in d['tournaments']:
		Nrnd = 1
		for rnd in d['tournaments'][trn]['roundList']:
			s  = ""  # gather output String of this round : game pairings with their results, and the current standings below it.
			s += GEOL
			s += f"### Round {Nrnd} -------------------------------------------------"
			s += GEOL
			s += GEOL
			aGame = []  # gather games : print finally
			maxW = 0
			maxB = 0
			for game in rnd:
				d2 = rtDplayersData(game)

				# determine longest White / Black name : to print nice table
				Lw = len(d2['nameWhite'])
				maxW = Lw if Lw > maxW else maxW
				Lb = len(d2['nameBlack'])
				maxB = Lb if Lb > maxB else maxB

				aGame.append( ( d2['nameWhite'], d2['nameBlack'], d2['result'] ) )
				if d2['nameWhite'] not in dStanding:
					dStanding.update({ d2['nameWhite']: {'Npoint':0, 'Ngame':0, 'Nprc':0, 'rating':0} })
				if d2['nameBlack'] not in dStanding:
					dStanding.update({ d2['nameBlack']: {'Npoint':0, 'Ngame':0, 'Nprc':0, 'rating':0} })

				dStanding[d2['nameWhite']]['Npoint'] += rtPoint(d2['result'][0:1])
				dStanding[d2['nameWhite']]['Ngame']  += 1
				dStanding[d2['nameWhite']]['Nprc']    = rtPRC(dStanding[d2['nameWhite']])
				dStanding[d2['nameWhite']]['rating']  = d2['ratingWhite']

				dStanding[d2['nameBlack']]['Npoint'] += rtPoint(d2['result'][2:3])
				dStanding[d2['nameBlack']]['Ngame']  += 1
				dStanding[d2['nameBlack']]['Nprc']    = rtPRC(dStanding[d2['nameBlack']])
				dStanding[d2['nameBlack']]['rating']  = d2['ratingBlack']

			for game in aGame:
				s += game[0].ljust(maxW) +" - "+ game[1].ljust(maxB) +" "+ game[2]
				s += GEOL

			# sort order : percentage > [#] games > name (reversed alpha)
			d3 = OrderedDict(sorted(dStanding.items(), key=lambda t: ( t[1]['Nprc'], 100-t[1]['Ngame'], t[0] ), reverse=True))
			nameMax = max([maxW,maxB])
			n = 1  # ranking

			s += GEOL
			s += sHeader
			s += " rating" if GTFshowRating else ""
			s += GEOL
			s += "--------------------------------------------"
			s += "-------" if GTFshowRating else ""
			s += GEOL

			for name in d3:
				s += str(n).rjust(2)
				s += ". "
				s += name.ljust(nameMax+3)
				s += rtPoint2(d3[name]['Npoint']).rjust(5)
				s += str(d3[name]['Ngame']).rjust(4)
				s += str(rtPRC(d3[name])).rjust(8)
				s += str(d3[name]['rating']).rjust(7) if GTFshowRating else ""
				s += GEOL

				n += 1

			# output String of this round : results and current standings
			print(s)

			Nrnd += 1


